<?php

class FastLoginPlugin extends Gdn_Plugin {

    const providerKey = 'Fast';

    public function base_connectData_handler($sender, $args) {
        if (!isset($args[0]) || $args[0] != 'fast') {
            return;
        }

        $request = Gdn::request();

        $challengeId = $request->get('challengeId');
        $oth = $request->get('oth');
        $identifier = $request->get('identifier');

        if ($identifier && $challengeId && $oth) {

            $proxyRequest = new ProxyRequest();
            $proxyRequest->request([
                'URL' => 'https://api.fast.co/api/verify/',
                'Method' => 'GET'
            ], [
                'challengeId' => $challengeId,
                'oth' => $oth,
                'identifier' => $identifier,
                'key' => c('FastLogin.AppKey'),
                'secret' => c('FastLogin.AppSecret')
            ]);

            $json = json_decode($proxyRequest->body(), true);

            if (isset($json['success']) && $json['success'] === true) {

                $form = $sender->Form;
                $form->setFormValue('UniqueID', $identifier);
                $form->setFormValue('Provider', self::providerKey);
                $form->setFormValue('ProviderName', self::providerKey);
                $form->setFormValue('Email', $identifier);
                $form->setFormValue('Target', c('FastLogin.Redirect', '/'));

                $sender->setData('Verified', true);

                $this->EventArguments['Form'] = $form;
                $this->fireEvent('AfterConnectData');

            } else {
                throw permissionException();
            }

        } else {
            throw permissionException();
        }
    }


    public function entryController_signIn_handler($sender) {
        if (!isset($sender->Data['Methods'])) {
            return;
        }
        $sender->Data['Methods'][] = [
            'Name' => 'Fast',
            'SignInHtml' => self::signInButton()
        ];
    }


    public function base_signInIcons_handler($sender) {
        echo ' '.self::signInButton(/*'small'*/).' ';
    }


    public function base_beforeSignInButton_handler($sender) {
        echo ' '.self::signInButton().' ';
    }


    public static function signInButton($type = 'button') {
        return '<fast-button></fast-button>';
    }


    public function base_render_before($sender) {
        if (isset($sender->Head)) {
            $sender->Head->addString('<style type="text/css">.MultipleEntryMethods .Methods{min-width:300px;}</style>');
        }
        saveToConfig('Garden.SignIn.Popup', false, false);
    }


    public function base_afterBody_handler($sender) {
        if (Gdn::session()->isValid()) {
            return;
        }

        $position = ['left', 'right', 'topleft', 'topright', 'none'];
        $iconid = c('FastLogin.IconID');

        $icon = (bool)$iconid ? urlencode($iconid) : $position[(int)c('FastLogin.IconPosition', 0)];

        echo '<script src="https://js.fast.co/button.js?key='.urlencode(c('FastLogin.AppKey')).'&icon='.$icon.'"></script>';
    }


    public function socialController_fastlogin_create($sender) {
        $sender->permission('Garden.Settings.Manage');

        $conf = new ConfigurationModule($sender);
        $conf->initialize([
            'FastLogin.IconPosition' => [
                'Control' => 'dropdown',
                'Items' => ['bottom left', 'bottom right', 'top left', 'top right', 'disabled'],
                'LabelCode' => 'Icon Position',
                'Description' => 'Choose the position of the floating Fast icon. Choose "disabled" to hide it.'
            ],
            'FastLogin.IconID' => [
                'LabelCode' => 'Icon ID',
                'Description' => 'Enter the ID of an element you want the Fast icon to be anchored to. Leave empty to use the option above.'
            ],
            'FastLogin.Redirect' => [
                'LabelCode' => 'Sign In Redirect',
                'Description' => 'Enter the route of a page you want users to be redirected to after signing in with Fast (e.g. "/categories").'
            ],
            'FastLogin.AppKey' => [
                'LabelCode' => 'Fast App Key',
                'Options' => ['class' => 'InputBox BigInput']
            ],
            'FastLogin.AppSecret' => [
                'LabelCode' => 'Fast App Secret',
                'Options' => ['class' => 'InputBox BigInput']
            ]
        ]);

        $sender->setData('Title', sprintf(t('%s Settings'), 'Fast'));
        $sender->setData('Description', sprintf(
            t('FastLogin.Description.NoConfig', 'At first you need to <a href="https://fast.co/developers/new">create a new App</a>. Enter this URL when asked: <code>%s</code>.<br>After that, <a href="https://fast.co/developers">look up</a> the "Key" and the "Secret" and save them here.<br><br><strong>If key and secret are already present, Fast has been automatically configured for you and you can skip these steps.</strong>'),
            Gdn::request()->url('entry/connect/fast', true)
        ));
        $conf->renderAll();
    }


    public function setup() {
        $sessionUser = Gdn::session()->User;

        if (c('FastLogin.AppKey') || c('FastLogin.AppSecret') || !$sessionUser->Admin) {
            return;
        }

        $proxyRequest = new ProxyRequest();
        $proxyRequest->request([
            'URL' => 'https://api.fast.co/api/enroll/',
            'Method' => 'GET'
        ], [
            'redirect' => Gdn::request()->url('entry/connect/fast', true),
            'platform' => 'Vanilla',
            'version' => APPLICATION_VERSION,
            'identifier' => $sessionUser->Email
        ]);

        $json = json_decode($proxyRequest->body(), true);

        if (isset($json['status']) && $json['status'] === 200) {
            saveToConfig('FastLogin.AppKey', $json['key']);
            saveToConfig('FastLogin.AppSecret', $json['secret']);
        }

    }

}
